//
//  MTS2MP4AppDelegate.m
//  MTS2MP4
//
//  Created by Robert McLeay on 14/06/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "MTS2MP4AppDelegate.h"
#import "MyPrefsWindowController.h"

@implementation MTS2MP4AppDelegate

+ (void)initialize {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSDictionary *appDefaults = [NSDictionary dictionaryWithObject:@"0.75" forKey:@"encodeQuality"];
	[defaults registerDefaults:appDefaults];
	
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"showMsgAtStartup"]) {
		[self showErrorMsg:@"Welcome and Congratulations. You've triggered an Easter Egg!"];
	}
	
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"showMsgAtStartup"]) {
		[self showErrorMsg:@"Welcome and Congratulations. You've triggered an Easter Egg!"];
	}
	
}

- (IBAction)openPreferencesWindow:(id)sender
{
	[[MyPrefsWindowController sharedPrefsWindowController] showWindow:nil];
}

// Handle a file dropped on the dock icon
- (void)application:(NSApplication *)sender openFiles:(NSArray *)filenames
{
	
	//[myProgress setUsesThreadedAnimation:YES];
	[myProgress setIndeterminate:YES];
	[myProgress startAnimation:self];
	[btnCancel setEnabled:YES];
	
	if (!running_jobs) {
		file_array = [[NSArray alloc] initWithArray:filenames];
		file_array_pos = 0;
		
		[lblProgress setStringValue:[NSString stringWithFormat:@"Converting file %i of %i: %@", file_array_pos+1, 
									 [file_array count], [file_array objectAtIndex:file_array_pos]]];
		
		[self convertMTSFile:[file_array objectAtIndex:file_array_pos]];
	}
	
}

- (IBAction)cancelJob:(id)sender
{	
	if ([current_task isRunning]) {
		[[NSNotificationCenter defaultCenter] removeObserver:self];
		[current_task terminate];
	}
	[btnCancel setEnabled:NO];
	[myProgress stopAnimation:self];
	[lblProgress setStringValue:@"Job Cancelled. Drag MTS file(s) to Dock icon to start converting..."];
	
}

- (void)showErrorMsg:(NSString*)errorStr {
	NSAlert *nsa = [[[NSAlert alloc] init] autorelease];
	[nsa setMessageText:errorStr];
	[nsa setAlertStyle:NSWarningAlertStyle];
	[nsa runModal];
}

- (void) convertMTSFile:(NSString*)file {
		
	//Settings stuff
	NSString *resourcePath = [[NSBundle mainBundle] resourcePath];
	NSString *outputfile = [NSString stringWithFormat:@"%@.mp4", [file substringToIndex:([file length]-4)]];
	NSString *encodeQualityStr = [[NSUserDefaults standardUserDefaults] objectForKey:@"encodeQuality"];
	double encodeQuality = atof([encodeQualityStr UTF8String]);
	
	if (encodeQualityStr == NULL || encodeQuality < 0 || encodeQuality > 1) {
		encodeQuality = 0.75;
		[[NSUserDefaults standardUserDefaults] setObject:@"0.75" forKey:@"encodeQuality"];
	}
	
	//Task stuff
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	outputPipe = [NSPipe pipe];
    taskOutput = [outputPipe fileHandleForReading];
    current_task = [[NSTask alloc] init];
	
    [current_task setLaunchPath: [NSString stringWithFormat:@"%@/HandBrakeCLI", resourcePath]];
	NSArray *arguments = [NSArray arrayWithObjects:@"-i", file, @"-e", @"x264", @"-E", @"ca_aac", @"-q", encodeQualityStr, @"-o", outputfile, nil];

	[current_task setArguments:arguments];
	[current_task setStandardInput:[NSPipe pipe]]; //keep the log in the console
	[current_task setStandardOutput:outputPipe];
    [current_task setStandardError:outputPipe];
	
	[defaultCenter addObserver:self selector:@selector(taskCompleted:) name:NSTaskDidTerminateNotification object:current_task];
	[defaultCenter addObserver:self selector:@selector(taskDataAvailable:) name:NSFileHandleReadCompletionNotification object:taskOutput];

    [current_task launch];
	[taskOutput readInBackgroundAndNotify];

}


- (void)taskCompleted:(NSNotification *)notif
{
    int exitCode = [[notif object] terminationStatus];
    
    if (exitCode != 0)
        [self showErrorMsg:@"Error: Conversion failed."];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
  
	// See if there's another in the queue.
	file_array_pos++;
	if (file_array_pos < [file_array count]) {
		[lblProgress setStringValue:[NSString stringWithFormat:@"Converting file %i of %i: %@", file_array_pos+1, 
									 [file_array count], [file_array objectAtIndex:file_array_pos]]];
		[myProgress setIndeterminate:YES];
		[self convertMTSFile:[file_array objectAtIndex:file_array_pos]];
	} else {
		running_jobs = NO;
		file_array = [[NSArray alloc] init];
		file_array_pos = 0;
		
		[btnCancel setEnabled:NO];
		[myProgress stopAnimation:self];
		[myProgress setIndeterminate:YES];
		[lblProgress setStringValue:@"Job(s) Finished. Drag MTS file(s) to Dock icon to start converting..."];
	}
}

- (void)taskDataAvailable:(NSNotification *)notif
{
    NSData *incomingData = [[notif userInfo] objectForKey:NSFileHandleNotificationDataItem];
    if (incomingData && [incomingData length])
    {
        NSString *incomingText = [[NSString alloc] initWithData:incomingData encoding:NSASCIIStringEncoding];
		NSArray *splitText = [incomingText componentsSeparatedByString:@" "];
		NSLog(@"%@", incomingText);
		
		//Check for errors
		if ([incomingText length] > 15 && [[incomingText substringToIndex:15] isEqualToString:@"No title found."]) {
			[self showErrorMsg:@"Error: Corrupt input file."];
			[current_task terminate];
		}
		
        // Do whatever with incomingText, the string that has some text in it
		if ([[splitText objectAtIndex:0] isEqualToString:@"\rEncoding:"]) {
			if ([myProgress isIndeterminate]) {
				[myProgress setIndeterminate:NO];
				[myProgress setMaxValue:100];
			}
			[myProgress setDoubleValue:atof([[splitText objectAtIndex:5] UTF8String])];
		}
		
		[taskOutput readInBackgroundAndNotify];
        [incomingText release];
        return;
    }
}

@end
